# -*- coding: utf-8 -*-
"""
app.network.forms
~~~~~~~~~~~~~~~~~

This module defines the network blueprint forms.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
from flask_login import current_user
from flask_wtf import FlaskForm
from wtforms import (SelectField, StringField, TextAreaField,
                     SelectMultipleField, BooleanField, validators)
from .. import utils, models


def validate_tags(form, field):
    choices = dict(field.choices)
    for tag_id in field.data:
        tag_name = choices.get(tag_id)
        if tag_name == 'gateway':
            network = models.Network.query.get(form.network_id.data)
            existing_gateway = network.gateway()
            if existing_gateway is not None:
                raise validators.ValidationError(f'A gateway is already defined for network {network}: {existing_gateway}')


class NoValidateSelectField(SelectField):
    """SelectField with no choices validation

    By default a SelectField tries to validate the selected value
    against the list of choices. This is not possible when the choices
    are dynamically created on the browser side.
    """

    def pre_validate(self, form):
        pass


class NetworkForm(FlaskForm):
    scope_id = SelectField('Network Scope')
    vlan_name = StringField('Vlan name',
                            description='hostname must be 3-25 characters long and contain only letters, numbers and dash',
                            validators=[validators.InputRequired(),
                                        validators.Regexp(models.VLAN_NAME_RE)])
    vlan_id = NoValidateSelectField('Vlan id', choices=[])
    description = TextAreaField('Description')
    prefix = NoValidateSelectField('Prefix', choices=[])
    address = NoValidateSelectField('Address', choices=[])
    first_ip = NoValidateSelectField('First IP', choices=[])
    last_ip = NoValidateSelectField('Last IP', choices=[])
    admin_only = BooleanField('Admin only')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.scope_id.choices = utils.get_model_choices(models.NetworkScope, attr='name')


class HostForm(FlaskForm):
    name = StringField('Hostname',
                       description='hostname must be 2-20 characters long and contain only letters, numbers and dash',
                       validators=[validators.InputRequired(),
                                   validators.Regexp(models.HOST_NAME_RE)],
                       filters=[utils.lowercase_field])
    type = SelectField('Type', choices=utils.get_choices(('Virtual', 'Physical')))
    description = TextAreaField('Description')
    item_id = SelectField('Item', coerce=utils.coerce_to_str_or_none)
    network_id = SelectField('Network')
    # The list of IPs is dynamically created on the browser side
    # depending on the selected network
    ip = NoValidateSelectField('IP', choices=[])
    interface_name = StringField(
        'Interface name',
        description='name must be 2-20 characters long and contain only letters, numbers and dash',
        validators=[validators.InputRequired(), validators.Regexp(models.HOST_NAME_RE)],
        filters=[utils.lowercase_field])
    mac_id = SelectField('MAC', coerce=utils.coerce_to_str_or_none)
    tags = SelectMultipleField('Tags', coerce=utils.coerce_to_str_or_none,
                               validators=[validate_tags])

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.item_id.choices = utils.get_model_choices(models.Item, allow_none=True, attr='ics_id')
        if current_user.is_admin:
            network_query = models.Network.query
        else:
            network_query = models.Network.query.filter(models.Network.admin_only.is_(False))
        self.network_id.choices = utils.get_model_choices(models.Network, allow_none=False,
                                                          attr='vlan_name', query=network_query)
        self.mac_id.choices = utils.get_model_choices(models.Mac, allow_none=True, attr='address')
        self.tags.choices = utils.get_model_choices(models.Tag, allow_none=True, attr='name')
