# -*- coding: utf-8 -*-
"""
app.network.views
~~~~~~~~~~~~~~~~~

This module implements the network blueprint.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
import ipaddress
import sqlalchemy as sa
from flask import (Blueprint, render_template, jsonify, session,
                   redirect, url_for, request, flash, current_app)
from flask_login import login_required
from .forms import HostForm, NetworkForm
from ..extensions import db
from ..decorators import login_groups_accepted
from .. import models

bp = Blueprint('network', __name__)


@bp.route('/hosts')
@login_required
def list_hosts():
    return render_template('network/hosts.html')


@bp.route('/hosts/create', methods=('GET', 'POST'))
@login_groups_accepted('admin', 'create')
def create_host():
    # Try to get the network_id from the session
    # to pre-fill the form with the same network
    try:
        network_id = session['network_id']
    except KeyError:
        # No need to pass request.form when no extra keywords are given
        form = HostForm()
    else:
        form = HostForm(request.form, network_id=network_id)
    if form.validate_on_submit():
        network_id = form.network_id.data
        host = models.Host(name=form.name.data,
                           type=form.type.data,
                           description=form.description.data or None,
                           item_id=form.item_id.data)
        # The total number of tags will always be quite small
        # It's more efficient to retrieve all of them in one query
        # and do the filtering here
        all_tags = models.Tag.query.all()
        tags = [tag for tag in all_tags if str(tag.id) in form.tags.data]
        host.interfaces = [
            models.Interface(name=form.interface_name.data,
                             ip=form.ip.data,
                             network_id=network_id,
                             mac_id=form.mac_id.data,
                             tags=tags)
        ]
        current_app.logger.debug(f'Trying to create: {host!r}')
        db.session.add(host)
        try:
            db.session.commit()
        except sa.exc.IntegrityError as e:
            db.session.rollback()
            current_app.logger.warning(f'{e}')
            flash(f'{e}', 'error')
        else:
            flash(f'Host {host} created!', 'success')
        # Save network_id to the session to retrieve it after the redirect
        session['network_id'] = network_id
        return redirect(url_for('network.create_host'))
    return render_template('network/create_host.html', form=form)


@bp.route('/_retrieve_hosts')
@login_required
def retrieve_hosts():
    data = [(host.name,
             host.type,
             host.description,
             interface.name,
             interface.ip,
             str(interface.network))
            for host in models.Host.query.all()
            for interface in host.interfaces]
    return jsonify(data=data)


@bp.route('/_retrieve_available_ips/<int:network_id>')
@login_required
def retrieve_available_ips(network_id):
    try:
        network = models.Network.query.get(network_id)
    except sa.exc.DataError:
        current_app.logger.warning(f'Invalid network_id: {network_id}')
        data = []
    else:
        data = [str(address) for address in network.available_ips()]
    return jsonify(data=data)


@bp.route('/networks')
@login_required
def list_networks():
    return render_template('network/networks.html')


@bp.route('/_retrieve_networks')
@login_required
def retrieve_networks():
    data = [(str(network.scope),
             network.vlan_name,
             network.vlan_id,
             network.description,
             network.address,
             network.first_ip,
             network.last_ip,
             network.admin_only)
            for network in models.Network.query.all()]
    return jsonify(data=data)


@bp.route('/networks/create', methods=('GET', 'POST'))
@login_groups_accepted('admin', 'create')
def create_network():
    # Try to get the scope_id from the session
    # to pre-fill the form with the same network scope
    try:
        scope_id = session['scope_id']
    except KeyError:
        # No need to pass request.form when no extra keywords are given
        form = NetworkForm()
    else:
        form = NetworkForm(request.form, scope_id=scope_id)
    if form.validate_on_submit():
        scope_id = form.scope_id.data
        network = models.Network(scope_id=scope_id,
                                 vlan_name=form.vlan_name.data,
                                 vlan_id=form.vlan_id.data,
                                 description=form.description.data or None,
                                 address=form.address.data,
                                 first_ip=form.first_ip.data,
                                 last_ip=form.last_ip.data,
                                 admin_only=form.admin_only.data)
        current_app.logger.debug(f'Trying to create: {network!r}')
        db.session.add(network)
        try:
            db.session.commit()
        except sa.exc.IntegrityError as e:
            db.session.rollback()
            current_app.logger.warning(f'{e}')
            flash(f'{e}', 'error')
        else:
            flash(f'Network {network} created!', 'success')
        # Save scope_id to the session to retrieve it after the redirect
        session['scope_id'] = scope_id
        return redirect(url_for('network.create_network'))
    return render_template('network/create_network.html', form=form)


@bp.route('/_retrieve_vlan_and_prefix/<int:scope_id>')
@login_required
def retrieve_vlan_and_prefix(scope_id):
    try:
        scope = models.NetworkScope.query.get(scope_id)
    except sa.exc.DataError:
        current_app.logger.warning(f'Invalid scope_id: {scope_id}')
        data = {'vlans': [], 'prefixes': [],
                'selected_vlan': '', 'selected_prefix': ''}
    else:
        vlans = [vlan_id for vlan_id in scope.available_vlans()]
        prefixes = scope.prefix_range()
        default_prefix = current_app.config['NETWORK_DEFAULT_PREFIX']
        if default_prefix in prefixes:
            selected_prefix = default_prefix
        else:
            selected_prefix = prefixes[0]
        data = {'vlans': vlans,
                'prefixes': prefixes,
                'selected_vlan': vlans[0],
                'selected_prefix': selected_prefix}
    return jsonify(data=data)


@bp.route('/_retrieve_subnets/<int:scope_id>/<int:prefix>')
@login_required
def retrieve_subnets(scope_id, prefix):
    try:
        scope = models.NetworkScope.query.get(scope_id)
    except sa.exc.DataError:
        current_app.logger.warning(f'Invalid scope_id: {scope_id}')
        data = {'subnets': [], 'selected_subnet': ''}
    else:
        subnets = [subnet for subnet in scope.available_subnets(int(prefix))]
        data = {'subnets': subnets,
                'selected_subnet': subnets[0]}
    return jsonify(data=data)


@bp.route('/_retrieve_ips/<subnet>/<int:prefix>')
@login_required
def retrieve_ips(subnet, prefix):
    try:
        address = ipaddress.ip_network(f'{subnet}/{prefix}')
    except ValueError:
        current_app.logger.warning(f'Invalid address: {subnet}/{prefix}')
        data = {'ips': [], 'first': '', 'last': ''}
    else:
        hosts = [str(ip) for ip in address.hosts()]
        if len(hosts) > 15:
            first = hosts[9]
            last = hosts[-5]
        else:
            first = hosts[0]
            last = hosts[-1]
        data = {'ips': hosts,
                'selected_first': first,
                'selected_last': last}
    return jsonify(data=data)
